@@ -7,7 +7,9 @@ import numpy as np
 
 
 class AllowMoveStrategy(abc.ABC):
-    def start_new_component(self, initial_labels, attr, objective_func, comp_idx):
+    def start_new_component(
+        self, initial_labels, attr, objective_func, comp_idx  # noqa: ARG002
+    ):
         """
         This method should be called whenever a new connected component is
         clustered.
@@ -64,7 +66,6 @@ class AllowMoveAZP(AllowMoveStrategy):
             self.objective_val += diff
             return True
         else:
-
             return False
 
 
@@ -86,7 +87,6 @@ class AllowMoveAZPSimulatedAnnealing(AllowMoveStrategy):
             self.notify_move_made()
             return True
         else:
-
             prob = math.exp(-diff / self.t)
             move_allowed = random.random() < prob
             if move_allowed:
@@ -153,11 +153,9 @@ class AllowMoveAZPMaxPRegions(AllowMoveStrategy):
         Parameters
         ----------
         spatially_extensive_attr : numpy.ndarray
-            See corresponding argument in
-            ``region.max_p_regions.heuristics.MaxPRegionsHeu.fit_from_scipy_sparse_matrix``.
+             Values that will serve as domain for the threshold constraint
         threshold : numbers.Real or numpy.ndarray
-            See corresponding argument in
-            ``region.max_p_regions.heuristics.MaxPRegionsHeu.fit_from_scipy_sparse_matrix``
+             Value for the threshold constraint
         decorated_strategy : AllowMoveStrategy
             The ``AllowMoveStrategy`` related to the algorithms local search.
         """
