@@ -1,7 +1,6 @@
 import geopandas
 import libpysal
 import numpy
-import unittest
 
 from spopt.region import WardSpatial
 
@@ -12,9 +11,8 @@ pth = libpysal.examples.get_path("mexicojoin.shp")
 MEXICO = geopandas.read_file(pth)
 
 
-class TestWard(unittest.TestCase):
-    def setUp(self):
-
+class TestWard:
+    def setup_method(self):
         self.mexico = MEXICO.copy()
         self.mexico["count"] = 1
         self.attrs_name = [f"PCGDP{year}" for year in range(1950, 2010, 10)]
